#include <fstream>

#include "ArhReader.h"
#include "Mesh.h"
#include "MeshStructure.h"

namespace AMDiS {
  
  using namespace std;

  void ArhReader::read(string filename, Mesh *mesh)
  {
    FUNCNAME("ArhReader::read()");

    RefinementManager *refManager;
    switch (mesh->getDim()) {
    case 2:
      refManager = new RefinementManager2d();
      break;
    case 3:
      refManager = new RefinementManager3d();
      break;
    default:
      ERROR_EXIT("Should not happen!\n");
    }

    ifstream file;
    file.open(filename.c_str(), ios::in | ios::binary);

    string typeId = "";
    long nMacroElements = 0;
    long nValueVectors = 0;
    long nAllValues = 0;

    file.read(const_cast<char*>(typeId.data()), 4);
    file.read(reinterpret_cast<char*>(&nMacroElements), 4);
    file.read(reinterpret_cast<char*>(&nValueVectors), 4);
    file.read(reinterpret_cast<char*>(&nAllValues), 4);

    for (long i = 0; i < nMacroElements; i++) {
      long elIndex = 0;
      long nStructureCodes = 0;
      long codeSize = 0;

      file.read(reinterpret_cast<char*>(&elIndex), 4);
      file.read(reinterpret_cast<char*>(&nStructureCodes), 4);
      file.read(reinterpret_cast<char*>(&codeSize), 4);

      vector<unsigned long> structureCode(nStructureCodes);
      file.read(reinterpret_cast<char*>(&(structureCode[0])), 4 * nStructureCodes);

      MeshStructure elementStructure;
      elementStructure.init(structureCode, codeSize);
      elementStructure.fitMeshToStructure(mesh, refManager, false, false, elIndex);

      long nValuesPerVector = 0;
      file.read(reinterpret_cast<char*>(&nValuesPerVector), 4);

      for (long j = 0; j < nValuesPerVector * nValueVectors; j++) {
	double value = 0.0;
	file.read(reinterpret_cast<char*>(&value), 8);
      }
    }

    file.close();

    MSG("ARH file read from: %s\n", filename.c_str());
  }
}
